<?php
declare(strict_types=1);

namespace Flat101\Base\Service;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\Store;

/**
 *  GetCurrentCategoryService
 */
class GetCurrentCategoryService
{
    /** @var CategoryInterface */
    private $currentCategory;

    /** @var int|null */
    private $categoryId;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var Http */
    private $request;

    /**
     * @param CategoryRepositoryInterface $categoryRepository
     * @param Http $request
     */
    public function __construct(
        CategoryRepositoryInterface $categoryRepository,
        Http $request
    ) {
        $this->categoryRepository = $categoryRepository;
        $this->request = $request;
    }

    /**
     * @return int|null
     */
    public function getCategoryId(): ?int
    {
        if ($this->request->getFullActionName() == 'catalog_category_view') {
            if (!$this->categoryId) {
                $this->categoryId = (int)$this->request->getParam('id', false);
            }

            return $this->categoryId ?: (int)$this->request->getParam('entity_id', false);
        }

        return null;
    }

    /**
     * @return CategoryInterface|null
     */
    public function getCategory(): ?CategoryInterface
    {
        if ($this->request->getFullActionName() == 'catalog_category_view') {
            if (!$this->currentCategory) {
                $categoryId = $this->getCategoryId();

                if (!$categoryId) {
                    return null;
                }

                try {
                    $storeId = (int)$this->request->getParam('store', false);
                    $storeId = $storeId ?: (int)$this->request->getParam('store_id', Store::DEFAULT_STORE_ID);

                    $this->currentCategory = $this->categoryRepository->get($categoryId);
                    $this->currentCategory->setStoreId($storeId);
                } catch (NoSuchEntityException $e) {
                    return null;
                }
            }
        }

        return $this->currentCategory;
    }
}
